@@ -36,6 +36,7 @@
 
 ;;; Code:
 
+(require 'eshell)
 (require 'json)
 
 ;; User customization
@@ -84,6 +85,8 @@ is active, after every command."
 (defcustom pyvenv-virtualenvwrapper-python
   (or (getenv "VIRTUALENVWRAPPER_PYTHON")
       (executable-find "python")
+      (executable-find "py")
+      (executable-find "pythonw")
       "python")
   "The python process which has access to the virtualenvwrapper module.
 
@@ -139,6 +142,9 @@ This is usually the base name of `pyvenv-virtual-env'.")
 (defvar pyvenv-old-exec-path nil
   "The old exec path before the last activate.")
 
+(defvar pyvenv-old-eshell-path nil
+  "The old eshell path before the last activate.")
+
 ;;;###autoload
 (defun pyvenv-activate (directory)
   "Activate the virtual environment in DIRECTORY."
@@ -152,10 +158,12 @@ This is usually the base name of `pyvenv-virtual-env'.")
         python-shell-virtualenv-root directory)
   ;; Preserve variables from being overwritten.
   (let ((old-exec-path exec-path)
+        (old-eshell-path eshell-path-env)
         (old-process-environment process-environment))
     (unwind-protect
         (pyvenv-run-virtualenvwrapper-hook "pre_activate" pyvenv-virtual-env)
       (setq exec-path old-exec-path
+            eshell-path-env old-eshell-path
             process-environment old-process-environment)))
   (run-hooks 'pyvenv-pre-activate-hooks)
   (let ((new-directories (append
@@ -164,12 +172,20 @@ This is usually the base name of `pyvenv-virtual-env'.")
                             (list (format "%s/bin" directory)))
                           ;; Windows
                           (when (file-exists-p (format "%s/Scripts" directory))
-                            (list (format "%s/Scripts" directory))))))
+                            (list (format "%s/Scripts" directory)
+                                  ;; Apparently, some virtualenv
+                                  ;; versions on windows put the
+                                  ;; python.exe in the virtualenv root
+                                  ;; for some reason?
+                                  directory)))))
     (setq pyvenv-old-exec-path exec-path
+          pyvenv-old-eshell-path eshell-path-env
           pyvenv-old-process-environment process-environment
           ;; For some reason, Emacs adds some directories to `exec-path'
           ;; but not to `process-environment'?
           exec-path (append new-directories exec-path)
+          ;; set eshell path to same as exec-path
+          eshell-path-env (mapconcat 'identity exec-path ":")
           process-environment (append
                                (list
                                 (format "VIRTUAL_ENV=%s" directory)
@@ -199,15 +215,20 @@ This is usually the base name of `pyvenv-virtual-env'.")
   (when pyvenv-old-exec-path
     (setq exec-path pyvenv-old-exec-path
           pyvenv-old-exec-path nil))
+  (when pyvenv-old-eshell-path
+    (setq eshell-path-env pyvenv-old-eshell-path
+          pyvenv-old-eshell-path nil))
   (when pyvenv-virtual-env
     ;; Make sure this does not change `exec-path', as $PATH is
     ;; different
     (let ((old-exec-path exec-path)
+          (old-eshell-path eshell-path-env)
           (old-process-environment process-environment))
       (unwind-protect
           (pyvenv-run-virtualenvwrapper-hook "post_deactivate"
                                              pyvenv-virtual-env)
         (setq exec-path old-exec-path
+              eshell-path-env old-eshell-path
               process-environment old-process-environment)))
     (run-hooks 'pyvenv-post-deactivate-hooks))
   (setq pyvenv-virtual-env nil
@@ -362,12 +383,11 @@ CAREFUL! This will modify your `process-environment' and
     (with-temp-buffer
       (let ((tmpfile (make-temp-file "pyvenv-virtualenvwrapper-")))
         (unwind-protect
-            (progn
+            (let ((default-directory (pyvenv-workon-home)))
               (apply #'call-process
                      pyvenv-virtualenvwrapper-python
                      nil t nil
-                     "-c"
-                     "from virtualenvwrapper.hook_loader import main; main()"
+                     "-m" "virtualenvwrapper.hook_loader"
                      "--script" tmpfile
                      (if (getenv "HOOK_VERBOSE_OPTION")
                          (cons (getenv "HOOK_VERBOSE_OPTION")
@@ -379,7 +399,7 @@ CAREFUL! This will modify your `process-environment' and
                nil t nil))
           (delete-file tmpfile)))
       (goto-char (point-min))
-      (when (and (not (re-search-forward "\\(ImportError\\|ModuleNotFoundError\\): No module named '?virtualenvwrapper'?" nil t))
+      (when (and (not (re-search-forward "No module named '?virtualenvwrapper'?" nil t))
                  (re-search-forward "\n=-=-=\n" nil t))
         (let ((output (buffer-substring (point-min)
                                         (match-beginning 0))))
